"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const servie_1 = require("servie");
const pathToRegexp = require("path-to-regexp");
const node_1 = require("popsicle/dist/node");
/**
 * Process request using basic node.js transport (no normalization).
 */
const send = node_1.send({});
/**
 * Create a simple service to proxy requests.
 */
class Service {
    constructor(config) {
        this.re = config.path ? pathToRegexp(config.path, [], { strict: true }) : undefined;
        this.methods = config.methods ? new Set(config.methods.map(x => x.toLowerCase())) : undefined;
        this.url = config.url;
    }
    isMethod(method) {
        return this.methods ? this.methods.has(method.toLowerCase()) : true;
    }
    isPath(path) {
        return this.re ? this.re.test(path) : true;
    }
    matches(req) {
        return this.isMethod(req.method) && this.isPath(req.url);
    }
}
exports.Service = Service;
/**
 * Proxy any number of paths to services.
 */
function proxy(configs) {
    const services = configs.map(config => new Service(config));
    return (req, next) => __awaiter(this, void 0, void 0, function* () {
        for (const service of services) {
            if (service.matches(req)) {
                const proxyReq = node_1.request(`${service.url}${req.url}`, {
                    method: req.method,
                    headers: req.headers,
                    body: req.body,
                    trailer: req.trailer
                });
                const res = yield send(proxyReq);
                return new servie_1.Response({
                    statusCode: res.statusCode,
                    statusMessage: res.statusMessage,
                    headers: res.headers,
                    body: res.body,
                    trailer: res.trailer
                });
            }
        }
        return next(); // 404.
    });
}
exports.proxy = proxy;
//# sourceMappingURL=index.js.map